/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.page;

import java.io.IOException;
import java.util.Collection;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;
import org.apache.struts.taglib.TagUtils;

/**
 * TODO documentation
 * 
 * @author rlogiacco
 * @jsp.tag name="paginate" body-content="JSP"
 *          tei-class="net.smartlab.web.page.PaginateTei" display-name=""
 *          description=""
 */
public class PaginateTag extends AbstractTag {

	/* # net.smartlab.web.page.PaginateTei tei */
	/**
	 * TODO documentation Comment for <code>PARAMETER</code>
	 */
	public final static String PARAMETER = "net.smartlab.web.page";

	/**
	 * TODO documentation Comment for <code>paginator</code>
	 * 
	 * @directed true
	 */
	private Paginator paginator = null;

	/**
	 * TODO documentation Comment for <code>name</code>
	 */
	private String name = null;

	/**
	 * TODO documentation
	 */
	private String property = null;

	/**
	 * TODO documentation Comment for <code>name</code>
	 */
	private String parameter;

	/**
	 * TODO documentation Comment for <code>href</code>
	 */
	private String href = "";

	/**
	 * TODO documentation Comment for <code>form</code>
	 */
	private String form = null;

	/**
	 * TODO documentation Comment for <code>size</code>
	 */
	private int size = Paginator.UNLIMITED_ITEMS;

	/**
	 * TODO documentation Comment for <code>pages</code>
	 */
	private int pages = Paginator.UNLIMITED_PAGES;


	/**
	 * @see javax.servlet.jsp.tagext.Tag#doStartTag()
	 */
	public int doStartTag() throws JspException {
		String index = ((HttpServletRequest)context.getRequest()).getParameter(parameter);
		Object attribute = TagUtils.getInstance().lookup(context, name, property, "request");
		if (attribute instanceof Paginator) {
			this.paginator = (Paginator)attribute;
		} else if (attribute instanceof Collection) {
			this.paginator = new CollectionPaginator((Collection)attribute);
			String attributeName = name;
			if (property != null) {
				attributeName = property;
			}
			context.setAttribute(attributeName, this.paginator, PageContext.PAGE_SCOPE);
		}
		context.setAttribute("paginator", this.paginator, PageContext.PAGE_SCOPE);
		if (size == Paginator.UNLIMITED_ITEMS) {
			paginator.setPageSize(paginator.getCount());
		} else {
			paginator.setPageSize(size);
		}
		paginator.setPages(pages);
		if (index != null && index.length() > 0) {
			paginator.setPage(Integer.parseInt(index));
		} else {
			paginator.setPage(1);
		}
		if (form != null) {
			try {
				context.getOut().println("<input type='hidden' name='" + parameter + "'/>");
				context.getOut().println("<script>");
				context.getOut().println("  function paginate(page) {");
				context.getOut().println("    document.forms['" + form + "'].elements['" + parameter + "'].value = page;");
				context.getOut().println("    document.forms['" + form + "'].action = '';");
				context.getOut().println("    document.forms['" + form + "'].submit();");
				context.getOut().println("  }");
				context.getOut().println("</script>");
			} catch (IOException ioe) {
				throw new JspTagException(ioe);
			}
		}
		return EVAL_BODY_INCLUDE;
	}

	/**
	 * TODO documentation
	 * 
	 * @param name
	 * @jsp.attribute required="true" rtexprvalue="false" type="String"
	 *                description="Name of the attribute containing the
	 *                paginator instance"
	 */
	public void setName(String name) {
		this.name = name;
		this.parameter = PARAMETER + '.' + name;
	}

	public void setProperty(String property) {
		this.property = property;
		if (property != null) {
			this.parameter = PARAMETER + '.' + name + "." + property;
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @return
	 */
	protected String getHref() {
		if (form == null) {
			String query = ((HttpServletRequest)context.getRequest()).getQueryString();
			if (query != null) {
				int start = query.indexOf(parameter + '=');
				int end = query.length();
				if (start > -1) {
					end = query.indexOf("&", start);
					if (end > -1) {
						query = query.substring(0, start) + query.substring(end + 1) + '&' + parameter + '=';
					} else {
						query = query.substring(0, start + parameter.length() + 1);
					}
				} else {
					query = query + '&' + parameter + '=';
				}
			} else {
				query = parameter + '=';
			}
			return href + '?' + query;
		} else {
			return href.substring(1);
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param href
	 * @jsp.attribute required="false" rtexprvalue="true" type="String"
	 *                description="The url where pagination actions are
	 *                submitted: leave blank or not specified to use the same
	 *                page"
	 */
	public void setHref(String href) {
		this.href = href;
	}

	/**
	 * TODO documentation
	 * 
	 * @param form
	 * @jsp.attribute required="false" rtexprvalue="true" type="String"
	 *                description="The name of form to use to vehicle pagination
	 *                informations"
	 */
	public void setForm(String form) {
		this.form = form;
	}

	/**
	 * TODO documentation
	 * 
	 * @param size
	 * @jsp.attribute required="false" rtexprvalue="true" type="int"
	 *                description="Number of items per page"
	 */
	public void setSize(int size) {
		this.size = size;
	}

	/**
	 * TODO documentation
	 * 
	 * @param pages
	 * @jsp.attribute required="false" rtexprvalue="true" type="int"
	 *                description="Maximum number of pages displayed in a row"
	 */
	public void setPages(int pages) {
		this.pages = pages;
	}

	/**
	 * @see net.smartlab.web.page.AbstractTag#getPaginator()
	 */
	protected Paginator getPaginator() {
		return paginator;
	}

	/**
	 * Checks if the pagination involves a form. Pagination with forms uses an
	 * hidden field to pass pagination infos, otherwise a request parameter is
	 * used.
	 * 
	 * @return <code>true</code> if the pagination involves a form,
	 *         <code>false</code> otherwise.
	 */
	protected boolean isForm() {
		return form != null;
	}
}
